import { IncomingMessage } from "node:http";
import {
	BadRequestException,
	Body,
	Controller,
	DefaultValuePipe,
	Delete,
	Get,
	HttpCode,
	HttpStatus,
	Param,
	ParseIntPipe,
	ParseUUIDPipe,
	Post,
	Query,
	Req,
	Request,
	Res,
	Response,
	StreamableFile,
	UseGuards,
} from "@nestjs/common";
import { CreateFileTypeDto } from "apps/backend/src/app/files/files.dto";
import { AdminGuard, InsertAdminState } from "../auth/auth.guard";
import { FilesService } from "./files.service";

@Controller("files")
export class FilesController {
	constructor(private readonly filesService: FilesService) {}

	@HttpCode(HttpStatus.OK)
	@UseGuards(InsertAdminState)
	@Post("new")
	async saveFile(@Req() req: IncomingMessage, @Res() res: Response) {
		let fileBuffer: Buffer = Buffer.from([]);
		req.on("data", (chunk: Buffer) => {
			fileBuffer = Buffer.concat([fileBuffer, chunk]);
		});

		req.on("end", async () => {
			try {
				console.log(fileBuffer);
				const _fileName = req.headers["file_name"] as string;
				const _groupId = req.headers["group_id"] as string;
				const _uploadedBy = req.headers["uploaded_by"] as string;
				const _machineId = req.headers["machine_id"];
				const _isDocumentation = req.headers["is_documentation"] as string;
				const _isRestricted = req.headers["is_restricted"] as string;
				const _isAdmin = Boolean(req.headers["is_admin"] as string | boolean);
				console.log(
					_fileName,
					_groupId,
					_uploadedBy,
					_machineId,
					_isDocumentation,
					_isRestricted,
					_isAdmin,
				);

				// Vérifier que les en-têtes nécessaires sont présents
				if (!_fileName || !_machineId || !_uploadedBy) {
					throw new BadRequestException("Header(s) manquant(s)");
				}
				console.log("Header found !");
				const machineId = Array(_machineId);

				const Params = new Map()
					.set("groupId", null)
					.set("fileName", _fileName.toString())
					.set("uploadedBy", String(_uploadedBy))
					.set("machineId", Array(machineId))
					.set("isDocumentation", false)
					.set("isRestricted", false);

				if (_groupId) Params.set("groupId", String(_groupId));

				console.log("Current params :\n", Params);

				//TODO Integrate a verification if the source is an admin, if that the case then it can define isDocumentation and isRestricted else throw in case of presence of those parameters.
				if (_isAdmin) {
					Params.set("isDocumentation", Boolean(_isDocumentation));
					Params.set("isRestricted", Boolean(_isRestricted));
				}

				console.log("Executing save procedure...");
				return (
					res
						// @ts-ignore
						.status(HttpStatus.CREATED)
						.send(await this.filesService.save(fileBuffer, Params))
				);
			} catch (err) {
				console.error(err);
				return (
					res
						// @ts-ignore
						.status(err.status || HttpStatus.INTERNAL_SERVER_ERROR)
						.send(err)
				);
			}
		});

		req.on("error", (err) => {
			return (
				res
					// @ts-ignore
					.status(err.status || HttpStatus.INTERNAL_SERVER_ERROR)
					.send(err)
			);
		});

		return;
	}

	@HttpCode(HttpStatus.FOUND)
	@Get("find")
	async findMany(
		@Query("limit", new DefaultValuePipe(20), ParseIntPipe) limit: number,
		@Query("offset", new DefaultValuePipe(0), ParseIntPipe) offset: number,
		@Query("search", new DefaultValuePipe("")) search: string,
	) {
		return this.filesService.search(limit, offset, search);
	}

	@HttpCode(HttpStatus.FOUND)
	@Get("types")
	async getTypes() {
		console.log("Performing request");
		return await this.filesService.getAllFilesTypes();
	}

	@HttpCode(HttpStatus.CREATED)
	@UseGuards(AdminGuard)
	@Post("types/new")
	async newType(@Body() body: CreateFileTypeDto) {
		return await this.filesService.createFileType(body.name, body.mime);
	}

	@HttpCode(HttpStatus.ACCEPTED)
	@UseGuards(AdminGuard)
	@Delete("types/:typeId")
	async delType(@Param(":typeId", ParseUUIDPipe) typeId: string) {
		return await this.filesService.removeFileType(typeId);
	}

	@HttpCode(HttpStatus.FOUND)
	@Get(":fileId")
	async getFile(@Param("fileId") fileId: string) {
		return await this.filesService.get(fileId);
	}

	@HttpCode(HttpStatus.OK)
	@UseGuards(AdminGuard)
	@Delete(":fileId")
	async deleteFile(@Param("fileId", ParseUUIDPipe) fileId: string) {
		return await this.filesService.deleteFile(fileId);
	}
}
